<?php

class App
{
    /** @var \Controller\ControllerInterface */
    private $controller = null;
    /** @var string */
    private $actionName = '';
    /** @var array */
    private $servicelocator = [
        'Index' => \Controller\IndexController::class,
    ];

    public function init(): void
    {
        $this->_initRequest();
        $this->controller->{$this->actionName}();
    }

    public function responseWith404(array $errorData = []): void
    {
        header('', true, 404);
        echo '<h1>Page not found</h1>';
        echo '<pre>';
        print_r($errorData);
        echo '</pre>';
        exit();
    }

    /**
     * @return string
     */
    private function _initRequest(): void
    {
        $request = trim($_SERVER['REQUEST_URI'] ?? '', '/');
        $requestUrlParts = explode('/', $request);
        $controllerName = ucfirst($requestUrlParts[0] ?: 'Index');

        if (isset($this->servicelocator[$controllerName])) {
            $controllerClass = $this->servicelocator[$controllerName];
            $this->controller = new $controllerClass();
        } else {
            $this->responseWith404([
                'message' => "Controller {$controllerName} not found"
            ]);
        }
        $this->actionName = lcfirst($requestUrlParts[1] ?? 'Index') . 'Action';

        if (!method_exists($this->controller, $this->actionName)) {
            $this->responseWith404([
                'message' => "Controller {$controllerName} has no action {$this->actionName}"
            ]);
        }
    }
}
